﻿using HeuristicLab.Clients.Access.Administration;
using HeuristicLab.Clients.Hive.WebJobManager.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace HeuristicLab.Clients.Hive.WebJobManager.ViewModels.User
{
    /// <summary>
    /// ViewModel containing User/Group/Role data
    /// </summary>
    public class UserViewModel
    {
        public HiveWebUser currentUser { get; set; }
        public string message { get; set; }
        public AccessAdministrationClient accessClient { get; set; }
        public List<Access.User> users { get; set; }
        public List<Access.Role> roles { get; set; }
        public List<Access.UserGroup> ugroups { get; set; }

        public Access.User SelectedUser { get; set; }
        public List<Access.UserGroup> SelectedUserSubscriptions { get; set; }
        public List<Access.Role> SelectedUserRoles { get; set; }
        public Access.UserGroup SelectedGroup { get; set; }
        public List<Access.UserGroupBase> SelectedGroupMembers { get; set; }
        public Access.Role SelectedRole { get; set; }
        public List<Access.UserGroupBase> SelectedRoleEnrolled { get; set; }

        /// <summary>
        /// Initialize viewmodel and set up connection
        /// </summary>
        /// <param name="ac">AccessAdminClient for connection</param>
        public UserViewModel(AccessAdministrationClient ac, HiveWebUser hwu)
        {
            currentUser = hwu;
            users = new List<Access.User>();
            accessClient = ac;

            SelectedUser = new Access.User();
            SelectedGroup = new Access.UserGroup();
            SelectedRole = new Access.Role();
            SelectedRole.Name = "";

            SelectedUserSubscriptions = new List<Access.UserGroup>();
            SelectedUserRoles = new List<Access.Role>();
            SelectedGroupMembers = new List<Access.UserGroupBase>();
            
            
        }
        /// <summary>
        /// Refreshes Users, Groups and Roles
        /// </summary>
        /// <returns></returns>
        public UserViewModel refreshAll()
        {
            refreshUsers();
            refreshGroups();
            refreshRoles();
            return this;
        }
        /// <summary>
        /// Refreshes only the Users
        /// </summary>
        /// <returns></returns>
        public UserViewModel refreshUsers()
        {
            accessClient.RefreshUsers();
            users = accessClient.Users.ToList();
            return this;
        }
        /// <summary>
        /// Refreshes only the Roles
        /// </summary>
        /// <returns></returns>
        public UserViewModel refreshRoles()
        {
            accessClient.RefreshRoles();
            roles = accessClient.Roles.ToList();
            return this;
        }
        /// <summary>
        /// Refreshes only the User Groups
        /// </summary>
        /// <returns></returns>
        public UserViewModel refreshGroups()
        {
            accessClient.RefreshUserGroups();
            ugroups= accessClient.Groups.ToList();
            return this;
        }
        /// <summary>
        /// Returns a user by its Id
        /// </summary>
        /// <param name="id">User Id</param>
        /// <returns></returns>
        public Access.User getUserById(Guid id)
        {
            return users.Find(x => x.Id == id);
        }
        /// <summary>
        /// Returns a group by its Id
        /// </summary>
        /// <param name="id">Group id</param>
        /// <returns></returns>
        public Access.UserGroup getGroupById(Guid id)
        {
            return ugroups.Find(x => x.Id == id);
        }
        /// <summary>
        /// Returns a role by its name
        /// </summary>
        /// <param name="name">Role name</param>
        /// <returns></returns>
        public Access.Role getRoleByName(string name)
        {
            return roles.Find(x => x.Name == name);
        }
    }
}
